{*********************************************************}
{                                                         }
{     TMyOpenReportDialog 1.0                             }
{                                                         }
{       A component displays a modal Windows dialog box   }
{        for selecting and opening QuickReport files.     }
{        This component is just like TOpenPictureDialog   }
{                                                         }
{         MinSu Kim (atint@netian.com), 1999              }
{                                                         }
{*********************************************************}
unit MyOpenReportDialog;

{$R-}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Dialogs,
  StdCtrls, Extctrls, Buttons, QRPrntr;

type

  TQRMyPreviewImage = class;

  TMyOpenReportDialog = class(TOpenDialog)
  private
    FQRPrinter: TQRPrinter;
	  FPreviewImage : TQRMyPreviewImage;
    FQuickRepPanel: TPanel;
    FQuickRepLabel: TLabel;
    FPreviewButton: TSpeedButton;
    FPaintPanel: TPanel;
    FPaintBox: TPaintBox;
    procedure PaintBoxPaint(Sender: TObject);
    procedure PreviewClick(Sender: TObject);
    procedure PreviewKeyPress(Sender: TObject; var Key: Char);
  protected
    procedure DoClose; override;
    procedure DoSelectionChange; override;
    procedure DoShow; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function Execute: Boolean; override;
  end;

  TQRMyPreviewImage = class(TGraphicControl)
  private
    FQRPrinter : TQRPrinter;
    FPageNumber : integer;
    FImageOK : boolean;
    procedure PaintPage;
    procedure SetPageNumber(Value : integer);
  public
    aMetaFile : TMetafile;
    Zoom : Integer;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Paint; override;
    property ImageOK : boolean read FImageOK write FImageOK;
    property PageNumber : integer read FPageNumber write SetPageNumber;
    property QRPrinter : TQRPrinter read FQRPrinter write FQRPrinter;
  end;

procedure Register;

implementation

uses Consts, Forms, CommDlg, Dlgs;

{$R MYOPENREPORTDIALOG.RES}
{ TQRMyPreviewImage }
constructor TQRMyPreviewImage.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Height := 250;
  Width := 250;
  Zoom := 100;
  QRPrinter := nil;
  ImageOK := false;
  FPageNumber := 1;
end;

destructor TQRMyPreviewImage.Destroy;
begin
  if ImageOK then
    aMetafile.Free;
  inherited Destroy;
end;

procedure TQRMyPReviewImage.PaintPage;
begin

end;

procedure TQRMyPreviewImage.Paint;
begin
  PaintPage;
end;

procedure TQRMyPreviewImage.SetPageNumber(Value : integer);
begin
  FPageNumber := Value;
  if assigned(aMetafile) then
    aMetafile.Free;
  aMetaFile := nil;
  if assigned(FQRPrinter) then
    aMetaFile := QRPrinter.GetPage(Value);
  ImageOK := aMetafile <> nil;
  PaintPage;
end;

{ TMyOpenReportDialog }
constructor TMyOpenReportDialog.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Filter := ' (*.' +cQRPDefaultExt + ')|*.' + cqrpDefaultExt;
  FQRPrinter := TQRPrinter.Create;
  FQuickRepPanel := TPanel.Create(Self);
  with FQuickRepPanel do
  begin
    Name := 'QuickRepPanel';
    Caption := '';
    SetBounds(204, 5, 169, 200);
    BevelOuter := bvNone;
    BorderWidth := 6;
    TabOrder := 1;
    FQuickRepLabel := TLabel.Create(Self);
    with FQuickRepLabel do
    begin
      Name := 'QuickRepLabel';
      Caption := '';
      SetBounds(6, 6, 157, 23);
      Align := alTop;
      AutoSize := False;
      Parent := FQuickRepPanel;
    end;
    FPreviewButton := TSpeedButton.Create(Self);
    with FPreviewButton do
    begin
      Name := 'PreviewButton';
      SetBounds(77, 1, 23, 22);
      Enabled := False;
      Glyph.LoadFromResourceName(HInstance, 'MYPREVIEWGLYPH');
      Hint := SPreviewLabel;
      ParentShowHint := False;
      ShowHint := True;
      OnClick := PreviewClick;
      Parent := FQuickRepPanel;
    end;
    FPaintPanel := TPanel.Create(Self);
    with FPaintPanel do
    begin
      Name := 'PaintPanel';
      Caption := '';
      SetBounds(6, 29, 157, 145);
      Align := alClient;
      BevelInner := bvRaised;
      BevelOuter := bvLowered;
      TabOrder := 0;
      Parent := FQuickRepPanel;
      FPaintBox := TPaintBox.Create(Self);
      FPaintBox.Color := Color;
      with FPaintBox do
      begin
        Name := 'PaintBox';
        SetBounds(0, 0, 153, 141);
        Align := alClient;
        OnDblClick := PreviewClick;
        OnPaint := PaintBoxPaint;
        Parent := FPaintPanel;
      end;
    end;
  end;
  FPreviewImage := TQRMyPreviewImage.Create(self);
  FPreviewImage.Parent := FPaintPanel;
  FPreviewImage.Visible := False;
  FPreviewImage.QRPrinter := FQRPrinter;
end;

destructor TMyOpenReportDialog.Destroy;
begin
  if FQRPrinter <> nil then FQRPrinter.Free;
  if FPreviewImage <> nil then FPreviewImage.Free;
  if FPaintBox <> nil then FPaintBox.Free;
  if FPaintPanel <> nil then FPaintPanel.Free;
  if FPreviewButton <> nil then FPreviewButton.Free;
  if FQuickRepLabel <> nil then FQuickRepLabel.Free;
  if FQuickRepPanel <> nil then FQuickRepPanel.Free;
  inherited Destroy;
end;

procedure TMyOpenReportDialog.DoSelectionChange;
var
  FullName: string;
  ValidPicture: Boolean;

  function ValidFile(const FileName: string): Boolean;
  begin
    Result := GetFileAttributes(PChar(FileName)) <> $FFFFFFFF;
  end;

begin
  inherited DoSelectionChange;
  FullName := FileName;
  ValidPicture := FileExists(FullName) and ValidFile(FullName);
  FPreviewImage.QRPrinter.Cleanup;
  if ValidPicture then
  try
    FQRPrinter.Load(Filename);
    FPreviewImage.PageNumber := 1;
    FQuickRepLabel.Caption := Format(SPictureLabel + SPictureDesc, [FPreviewImage.aMetaFile.Width,
      FPreviewImage.aMetaFile.Height]);
    FPreviewButton.Enabled := True;
  except
    ValidPicture := False;
  end;
  if not ValidPicture then
  begin
    FQuickRepLabel.Caption := SPictureLabel;
    FPreviewButton.Enabled := False;
    FPreviewImage.PageNumber := 1;
  end;
  FPaintBox.Invalidate;
end;

procedure TMyOpenReportDialog.DoClose;
begin
  inherited DoClose;
  { Hide any hint windows left behind }
  Application.HideHint;
end;

procedure TMyOpenReportDialog.DoShow;
var
  PreviewRect: TRect;
begin
  { Set preview area to entire dialog }
  GetClientRect(Handle, PreviewRect);
  { Move preview area to right of static area }
  PreviewRect.Left := GetStaticRect.Right;
  Inc(PreviewRect.Top, 4);
  FQuickRepPanel.BoundsRect := PreviewRect;
  FPreviewButton.Left := FPaintPanel.BoundsRect.Right - FPreviewButton.Width - 2;
  FPreviewImage.QRPrinter.Cleanup;
  FPreviewImage.PageNumber := 1;
  FQuickRepPanel.ParentWindow := Handle;
  inherited DoShow;
end;

function TMyOpenReportDialog.Execute;
begin
  if NewStyleControls and not (ofOldStyleDialog in Options) then
    Template := 'MYDLGTEMPLATE' else
    Template := nil;
  Result := inherited Execute;
end;

procedure TMyOpenReportDialog.PaintBoxPaint(Sender: TObject);
var
  DrawRect: TRect;
  SNone: string;
begin
  with TPaintBox(Sender) do
  begin
    Canvas.Brush.Color := Color;
    DrawRect := ClientRect;
    if FPreviewImage.ImageOK then
    begin
      with DrawRect do
        if (FPreviewImage.aMetaFile.Width > Right - Left) or (FPreviewImage.aMetaFile.Height > Bottom - Top) then
        begin
          if FPreviewImage.aMetaFile.Width > FPreviewImage.aMetaFile.Height then
            Bottom := Top + MulDiv(FPreviewImage.aMetaFile.Height, Right - Left, FPreviewImage.aMetaFile.Width)
          else
            Right := Left + MulDiv(FPreviewImage.aMetaFile.Width, Bottom - Top, FPreviewImage.aMetaFile.Height);
          Canvas.StretchDraw(DrawRect, FPreviewImage.aMetaFile)
        end
        else
          with DrawRect do
            Canvas.Draw(Left + (Right - Left - FPreviewImage.aMetaFile.Width) div 2, Top + (Bottom - Top -
              FPreviewImage.aMetaFile.Height) div 2, FPreviewImage.aMetaFile );
    end
    else
    begin
      with DrawRect, Canvas do
      begin
        SNone := srNone;
        TextOut(Left + (Right - Left - TextWidth(SNone)) div 2, Top + (Bottom -
          Top - TextHeight(SNone)) div 2, SNone);
      end;
      Canvas.FillRect(DrawRect);
    end;
  end;
end;

procedure TMyOpenReportDialog.PreviewClick(Sender: TObject);
var
  PreviewForm: TForm;
  Panel: TPanel;
begin
  if not FPreviewImage.ImageOK then Exit;
  PreviewForm := TForm.Create(Self);
  with PreviewForm do
  try
    Name := 'PreviewForm';
    Caption := SPreviewLabel;
    BorderStyle := bsSizeToolWin;
    KeyPreview := True;
    Position := poScreenCenter;
    OnKeyPress := PreviewKeyPress;
    Panel := TPanel.Create(PreviewForm);
    with Panel do
    begin
      Name := 'Panel';
      Caption := '';
      Align := alClient;
      BevelOuter := bvNone;
      BorderStyle := bsSingle;
      BorderWidth := 5;
      Color := clWindow;
      Parent := PreviewForm;
      with TImage.Create(PreviewForm) do
      begin
        Name := 'Image';
        Caption := '';
        Align := alClient;
        Stretch := True;
        Picture.Assign(FPreviewImage.aMetaFile);
        Parent := Panel;
      end;
    end;
    if FPreviewImage.ImageOK then
    begin
      ClientWidth := FPreviewImage.aMetaFile.Width + (ClientWidth - Panel.ClientWidth)+ 10;
      ClientHeight := FPreviewImage.aMetaFile.Height + (ClientHeight - Panel.ClientHeight) + 10;
    end;
    ShowModal;
  finally
    Free;
  end;
end;

procedure TMyOpenReportDialog.PreviewKeyPress(Sender: TObject; var Key: Char);
begin
  if Key = #27 then TForm(Sender).Close;
end;

procedure Register;
begin
  RegisterComponents('MinSu', [TMyOpenReportDialog]);
end;

end.
